self.importScripts('../indexeddb.min.js');
self.importScripts('../sodium.min.js');

// https://developer.mozilla.org/en-US/docs/Web/API/ServiceWorkerGlobalScope
self.addEventListener('push', function (event) {
	var notificationTitle = 'Synology';
	var notificationOptions = {
		body: '',
		icon: './images/notify_system.png',
		data: {
			url: 'https://www.synology.com/'
		}
	};

	if (event.data) {
		var dataText = event.data.text();
		if (dataText == '') {
			return;
		}

		var data = JSON.parse(dataText);

		if (typeof(data.event_category) != 'undefined') {
			notificationTitle = data.event_category;
		}
		if (typeof(data.notify_url) != 'undefined') {
			notificationOptions.data.url = data.notify_url;
		}

		if (data.event_category == 'Chat') {
			notificationOptions.icon = './images/notify_chat.png';
		} else if (data.event_category == 'Calendar') {
			notificationOptions.icon = './images/notify_calendar.png';
		} else if (data.event_category == 'MailClient') {
			notificationOptions.icon = './images/notify_mailclient.png';
		} else if (data.event_category == 'Spreadsheet') {
			notificationOptions.icon = './images/notify_office.png';
		}

		if (typeof(data.encrypted_data) != 'undefined') {
			// package is encrypted and device has shared key
			try {
				var payload = JSON.parse(SYNO.SAS.sodium.to_string(SYNO.SAS.sodium.from_base64(data.encrypted_data)));

				event.waitUntil(
					idbkeyval.get(payload.browser_key).then(function(b64sharedkey) {
						if (typeof(b64sharedkey) == 'undefined') {
							notificationOptions.body = data.raw_data || "Reinstall push service";
						} else {
							var sharedkey = SYNO.SAS.sodium.from_base64(b64sharedkey).subarray(0, SYNO.SAS.sodium.crypto_kx_SECRETKEYBYTES);
							var nonce = SYNO.SAS.sodium.from_base64(payload.nonce);
							var ciphermsg = SYNO.SAS.sodium.from_base64(payload.ciphertext);
							var message = SYNO.SAS.sodium.to_string(SYNO.SAS.sodium.crypto_secretbox_open_easy(ciphermsg, nonce, sharedkey));
							notificationOptions.body = message;
						}
						return self.registration.showNotification(notificationTitle, notificationOptions);
					})
				);
			} catch (e) {
				notificationOptions.body = data.raw_data || "Server Error";
				event.waitUntil(self.registration.showNotification(notificationTitle, notificationOptions));
			}
		} else if (typeof(data.raw_data) != 'undefined') {
			notificationOptions.body = data.raw_data;
			event.waitUntil(self.registration.showNotification(notificationTitle, notificationOptions));
		}
	}
});

self.addEventListener('notificationclick', function (event) {
	event.notification.close();

	var clickResponsePromise = Promise.resolve();
	if (event.notification.data && event.notification.data.url) {
		clickResponsePromise = clients.openWindow(event.notification.data.url);
	}

	event.waitUntil(Promise.all([clickResponsePromise]));
});
